@@ -23,6 +23,8 @@
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
 # IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
 import asyncio as aio
+import logging
+from typing import Any, Coroutine, Set
 from .message import BROADCAST_MAC, BROADCAST_SOURCE_ID
 from .msgtypes import *
 from .products import *
@@ -31,6 +33,13 @@ from functools import partial
 from math import floor
 import time, random, datetime, socket, ifaddr
 
+# prevent tasks from being garbage collected
+_BACKGROUND_TASKS: Set[aio.Task] = set()
+
+if sys.version_info[:2] < (3, 11):
+    from async_timeout import timeout as asyncio_timeout
+else:
+    from asyncio import timeout as asyncio_timeout
 
 # A couple of constants
 LISTEN_IP = "0.0.0.0"
@@ -40,6 +49,16 @@ DEFAULT_TIMEOUT = 0.5  # How long to wait for an ack or response
 DEFAULT_ATTEMPTS = 3  # How many time shou;d we try to send to the bulb`
 DISCOVERY_INTERVAL = 180
 DISCOVERY_STEP = 5
+MAX_UNSIGNED_16_BIT_INTEGER_VALUE = int("0xFFFF", 16)
+_LOGGER = logging.getLogger(__name__)
+
+
+def _create_background_task(coro: Coroutine) -> None:
+    """Create a background task that will not be garbage collected."""
+    global _BACKGROUND_TASKS
+    task = aio.create_task(coro)
+    _BACKGROUND_TASKS.add(task)
+    task.add_done_callback(_BACKGROUND_TASKS.discard)
 
 
 def mac_to_ipv6_linklocal(mac, prefix="fe80::"):
@@ -116,7 +135,7 @@ class Device(aio.DatagramProtocol):
         self.seq = 0
         # Key is the message sequence, value is (Response, Event, callb )
         self.message = {}
-        self.source_id = random.randint(0, (2 ** 32) - 1)
+        self.source_id = random.randint(0, (2**32) - 1)
         # Default callback for unexpected messages
         self.default_callb = None
         # And the rest
@@ -151,6 +170,25 @@ class Device(aio.DatagramProtocol):
         self.transport = transport
         self.register()
 
+    def error_received(self, exc: Exception) -> None:
+        """Method run when an error is received from the device.
+
+        This method is called in rare conditions, when the transport (e.g. UDP)
+        detects that a datagram could not be delivered to its recipient.
+        In many conditions though, undeliverable datagrams will be silently dropped.
+        """
+        _LOGGER.debug("%s: Error received: %s", self.ip_addr, exc)
+        # Clear the message queue since we know they are not going to be answered
+        # and there is no point in waiting for them
+        for entry in self.message.values():
+            response_type, myevent, callb = entry
+            if response_type != Acknowledgement:
+                if callb:
+                    callb(self, None)
+                if myevent:
+                    myevent.set()
+        self.message.clear()
+
     def datagram_received(self, data, addr):
         """Method run when data is received from the device
 
@@ -271,7 +309,7 @@ class Device(aio.DatagramProtocol):
             ack_requested=False,
             response_requested=False,
         )
-        xx = self.loop.create_task(self.fire_sending(msg, num_repeats))
+        _create_background_task(self.fire_sending(msg, num_repeats))
         return True
 
     async def try_sending(self, msg, timeout_secs, max_attempts):
@@ -305,7 +343,8 @@ class Device(aio.DatagramProtocol):
             if self.transport:
                 self.transport.sendto(msg.packed_message)
             try:
-                myresult = await aio.wait_for(event.wait(), timeout_secs)
+                async with asyncio_timeout(timeout_secs):
+                    await event.wait()
                 break
             except Exception as inst:
                 if attempts >= max_attempts:
@@ -345,7 +384,7 @@ class Device(aio.DatagramProtocol):
             response_requested=False,
         )
         self.message[msg.seq_num] = [Acknowledgement, None, callb]
-        xx = self.loop.create_task(self.try_sending(msg, timeout_secs, max_attempts))
+        _create_background_task(self.try_sending(msg, timeout_secs, max_attempts))
         return True
 
     # Usually used for Get messages, or for state confirmation after Set (hence the optional payload)
@@ -384,7 +423,7 @@ class Device(aio.DatagramProtocol):
             response_requested=True,
         )
         self.message[msg.seq_num] = [response_type, None, callb]
-        xx = self.loop.create_task(self.try_sending(msg, timeout_secs, max_attempts))
+        _create_background_task(self.try_sending(msg, timeout_secs, max_attempts))
         return True
 
     # Not currently implemented, although the LIFX LAN protocol supports this kind of workflow natively
@@ -421,7 +460,7 @@ class Device(aio.DatagramProtocol):
             response_requested=True,
         )
         self.message[msg.seq_num] = [response_type, None, callb]
-        xx = self.loop.create_task(self.try_sending(msg, timeout_secs, max_attempts))
+        _create_background_task(self.try_sending(msg, timeout_secs, max_attempts))
         return True
 
     #
@@ -599,13 +638,17 @@ class Device(aio.DatagramProtocol):
             mycallb = lambda x, y: mypartial(y)
         if value in on and not rapid:
             response = self.req_with_ack(
-                SetPower, {"power_level": 65535}, callb=mycallb
+                SetPower,
+                {"power_level": MAX_UNSIGNED_16_BIT_INTEGER_VALUE},
+                callb=mycallb,
             )
         elif value in off and not rapid:
             response = self.req_with_ack(SetPower, {"power_level": 0}, callb=mycallb)
         elif value in on and rapid:
-            response = self.fire_and_forget(SetPower, {"power_level": 65535})
-            self.power_level = 65535
+            response = self.fire_and_forget(
+                SetPower, {"power_level": MAX_UNSIGNED_16_BIT_INTEGER_VALUE}
+            )
+            self.power_level = MAX_UNSIGNED_16_BIT_INTEGER_VALUE
         elif value in off and rapid:
             response = self.fire_and_forget(SetPower, {"power_level": 0})
             self.power_level = 0
@@ -804,7 +847,7 @@ class Device(aio.DatagramProtocol):
         """Convenience to string method."""
         s = "Vendor: {}\n".format(self.vendor)
         s += indent + "Product: {}\n".format(
-            (self.product and product_map[self.product]) or "Unknown"
+            (self.product and products_dict[self.product]) or "Unknown"
         )
         s += indent + "Version: {}\n".format(self.version)
         return s
@@ -876,6 +919,14 @@ class Light(Device):
         self.hev_cycle_configuration = None
         self.last_hev_cycle_result = None
         self.effect = {"effect": None}
+        # Only used by a Lifx Switch. Will be populated with either True or False for each relay index if `get_rpower` called.
+        # At the moment we assume the switch to be 4 relays. This will likely work with the 2 relays switch as well, but only the first two values
+        # in this array will contain useful data.
+        self.relays_power = [None, None, None, None]
+        # Only used by a Lifx switch. Will be populated with an object containing the `haptic_duration_ms`, `backlight_on_color` and `backlight_off_color`
+        self.button_config = None
+        # Only used by a Lifx switch. Will be populated with an object containing `count`, `index`, `buttons_count` and `buttons`
+        self.button = None
 
     def get_power(self, callb=None):
         """Convenience method to request the power status from the device
@@ -916,7 +967,7 @@ class Light(Device):
         on = [True, 1, "on"]
         off = [False, 0, "off"]
         if value in on:
-            myvalue = 65535
+            myvalue = MAX_UNSIGNED_16_BIT_INTEGER_VALUE
         else:
             myvalue = 0
         mypartial = partial(self.resp_set_lightpower, power_level=myvalue)
@@ -1143,7 +1194,7 @@ class Light(Device):
         """Convenience method to start or stop the Move firmware effect on multizone devices.
 
         Compatible devices include LIFX Z, Lightstrip and Beam and can be identified by
-        checking if features_map[device.product]['multizone'] is True. Multizone devices
+        checking if products_dict[device.product].multizone is True. Multizone devices
         only have one firmware effect named "MOVE". The effect can be started and stopped
         without the device being powered on. The effect will not be visible if the
         device is a single uniform color.
@@ -1428,7 +1479,7 @@ class Light(Device):
         :returns: None
         :rtype: None
         """
-        if features_map[self.product]["hev"] is True:
+        if products_dict[self.product].hev is True:
             self.req_with_resp(GetHevCycle, StateHevCycle, callb=callb)
 
     def resp_set_hevcycle(self, resp):
@@ -1460,7 +1511,7 @@ class Light(Device):
         :returns: None
         :rtype: None
         """
-        if features_map[self.product]["hev"] is True:
+        if products_dict[self.product].hev is True:
             if rapid:
                 self.fire_and_forget(
                     SetHevCycle,
@@ -1489,7 +1540,7 @@ class Light(Device):
         :returns: None
         :rtype: None
         """
-        if features_map[self.product]["hev"] is True:
+        if products_dict[self.product].hev is True:
             self.req_with_resp(
                 GetHevCycleConfiguration, StateHevCycleConfiguration, callb=callb
             )
@@ -1522,7 +1573,7 @@ class Light(Device):
         :returns: None
         :rtype: None
         """
-        if features_map[self.product]["hev"] is True:
+        if products_dict[self.product].hev is True:
             if rapid:
                 self.fire_and_forget(
                     SetHevCycleConfiguration,
@@ -1552,7 +1603,7 @@ class Light(Device):
         :returns: None
         :rtype: None
         """
-        if features_map[self.product]["hev"] is True:
+        if products_dict[self.product].hev is True:
             self.req_with_resp(
                 GetLastHevCycleResult, StateLastHevCycleResult, callb=callb
             )
@@ -1649,6 +1700,219 @@ class Light(Device):
                 self.effect["palette_count"] = resp.palette_count
                 self.effect["palette"] = resp.palette
 
+    def get_rpower(self, relay_index=None, callb=None):
+        """Method will get the power state of all relays; or a single relay if value provided.
+
+        :param relay_index: The index of the relay to check power state for. If not provided, will loop through 4 relays
+        :type relay_index: int
+        :param callb: Callable to be used when the response is received.
+        :type callb: callable
+        :returns: The cached value
+        :rtype: int
+        """
+        mypartial = partial(self.resp_set_rpower)
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        if relay_index is not None:
+            payload = {"relay_index": relay_index}
+            response = self.req_with_resp(
+                GetRPower, StateRPower, payload, callb=mycallb
+            )
+        else:
+            for relay_index in range(4):
+                payload = {"relay_index": relay_index}
+                response = self.req_with_resp(
+                    GetRPower, StateRPower, payload, callb=mycallb
+                )
+        return self.relays_power
+
+    def set_rpower(self, relay_index, is_on, callb=None, rapid=False):
+        """Sets relay power for a given relay index
+
+        :param relay_index: The relay on the switch starting from 0.
+        :type relay_index: int
+        :param on: Whether the relay is on or not
+        :type is_on: bool
+        :param callb: Callable to be used when the response is received.
+        :type callb: callable
+        :param rapid: Whether to ask for ack (False) or not (True). Default False
+        :type rapid: bool
+        :returns: None
+        :rtype: None
+        """
+        level = 0
+        if is_on:
+            level = MAX_UNSIGNED_16_BIT_INTEGER_VALUE
+
+        payload = {"relay_index": relay_index, "level": level}
+        mypartial = partial(self.resp_set_rpower, relay_index=relay_index, level=level)
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        if not rapid:
+            self.req_with_resp(SetRPower, StateRPower, payload, callb=mycallb)
+        else:
+            self.fire_and_forget(SetRPower, payload)
+
+    def resp_set_rpower(self, resp, relay_index=None, level=None):
+        """Default callback for get_rpower/set_rpower"""
+        if relay_index != None and level != None:
+            self.relays_power[relay_index] = level == MAX_UNSIGNED_16_BIT_INTEGER_VALUE
+        elif resp:
+            # Current models of the LIFX switch do not have dimming capability, so the two valid values are 0 for off (False) and 65535 for on (True).
+            self.relays_power[resp.relay_index] = (
+                resp.level == MAX_UNSIGNED_16_BIT_INTEGER_VALUE
+            )
+
+    def get_button(self, callb=None):
+        """Method will get the state of all buttons
+
+        :param relay_index: The index of the relay to check power state for. If not provided, will loop through 4 relays
+        :type relay_index: int
+        :param callb: Callable to be used when the response is received.
+        :type callb: callable
+        :returns: The cached value
+        :rtype: int
+        """
+        mypartial = partial(self.resp_get_button)
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        payload = {}
+        response = self.req_with_resp(GetButton, StateButton, payload, callb=mycallb)
+
+    def set_button(self, callb=None, rapid=False):
+        raise Exception(
+            "SetButton isn't yet implemented as you can only set button actions to the same values as the LIFX app (ie you can't add custom callbacks), making it not that useful. Feel free to implement if you need this :)"
+        )
+        """ Sets button
+
+            :param callb: Callable to be used when the response is received.
+            :type callb: callable
+            :param rapid: Whether to ask for ack (False) or not (True). Default False
+            :type rapid: bool
+            :returns: None
+            :rtype: None
+        """
+
+        payload = {}
+        mypartial = partial(self.resp_get_button)
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        if not rapid:
+            self.req_with_resp(SetButton, StateButton, payload, callb=mycallb)
+        else:
+            self.fire_and_forget(SetButton, payload)
+
+    def resp_get_button(self, resp):
+        """Default callback for get_button/set_button"""
+        self.button = {
+            "count": resp.count,
+            "index": resp.index,
+            "buttons_count": resp.buttons_count,
+            "buttons": resp.buttons,
+        }
+
+    def get_button_config(self, callb=None):
+        """Method will get the button config
+
+        :param callb: Callable to be used when the response is received.
+        :type callb: callable
+        :returns: The cached value
+        :rtype: int
+        """
+        mypartial = partial(self.resp_get_button_config)
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        response = self.req_with_resp(
+            GetButtonConfig, StateButtonConfig, {}, callb=mycallb
+        )
+
+    def set_button_config(
+        self,
+        haptic_duration_ms: int,
+        backlight_on_color,
+        backlight_off_color,
+        callb=None,
+        rapid=False,
+    ):
+        """Sets button config
+
+        :param haptic_duration_ms: How many milliseconds the haptic vibration when the button is pressed should last
+        :type haptic_duration_ms: int
+        :param backlight_on_color: The color the backlight should be when a button is on
+        :type backlight_on_color: { "hue": int, "saturation": int, "brightness": int, "kelvin": int }
+        :param backlight_off_color: The color the backlight should be when a button is off
+        :type backlight_off_color: { "hue": int, "saturation": int, "brightness": int, "kelvin": int }
+        :param callb: Callable to be used when the response is received.
+        :type callb: callable
+        :param rapid: Whether to ask for ack (False) or not (True). Default False
+        :type rapid: bool
+        :returns: None
+        :rtype: None
+        """
+
+        payload = {
+            "haptic_duration_ms": haptic_duration_ms,
+            "backlight_on_color": backlight_on_color,
+            "backlight_off_color": backlight_off_color,
+        }
+        mypartial = partial(
+            self.resp_get_button_config,
+            haptic_duration_ms=haptic_duration_ms,
+            backlight_on_color=backlight_on_color,
+            backlight_off_color=backlight_off_color,
+        )
+        if callb:
+            mycallb = lambda x, y: (mypartial(y), callb(x, y))
+        else:
+            mycallb = lambda x, y: mypartial(y)
+
+        if not rapid:
+            self.req_with_resp(
+                SetButtonConfig, StateButtonConfig, payload, callb=mycallb
+            )
+        else:
+            self.fire_and_forget(SetButtonConfig, payload)
+
+    def resp_get_button_config(
+        self,
+        resp,
+        haptic_duration_ms=None,
+        backlight_on_color=None,
+        backlight_off_color=None,
+    ):
+        """Default callback for get_button_config/set_button_config"""
+        if (
+            haptic_duration_ms != None
+            and backlight_on_color != None
+            and backlight_off_color != None
+        ):
+            self.button_config = {
+                "haptic_duration_ms": haptic_duration_ms,
+                "backlight_on_color": backlight_on_color,
+                "backlight_off_color": backlight_off_color,
+            }
+        elif resp:
+            self.button_config = {
+                "haptic_duration_ms": resp.haptic_duration_ms,
+                "backlight_on_color": resp.backlight_on_color,
+                "backlight_off_color": resp.backlight_off_color,
+            }
+
     def get_accesspoint(self, callb=None):
         """Convenience method to request the access point available
 
@@ -1711,7 +1975,7 @@ class LifxDiscovery(aio.DatagramProtocol):
         self.transport = None
         self.loop = loop
         self.task = None
-        self.source_id = random.randint(0, (2 ** 32) - 1)
+        self.source_id = random.randint(0, (2**32) - 1)
         self.ipv6prefix = ipv6prefix
         self.discovery_interval = discovery_interval
         self.discovery_step = discovery_step
@@ -1724,7 +1988,7 @@ class LifxDiscovery(aio.DatagramProtocol):
             lambda: self, local_addr=(listen_ip, listen_port)
         )
 
-        self.task = self.loop.create_task(coro)
+        self.task = aio.create_task(coro)
         return self.task
 
     def connection_made(self, transport):
@@ -1793,7 +2057,7 @@ class LifxDiscovery(aio.DatagramProtocol):
             lambda: light, family=family, remote_addr=(remote_ip, remote_port)
         )
 
-        light.task = self.loop.create_task(coro)
+        light.task = aio.create_task(coro)
 
     def discover(self):
         """Method to send a discovery message"""
@@ -1866,7 +2130,7 @@ class LifxScan:
             lifx_discovery = LifxDiscovery(self.loop, manager)
             discoveries.append(lifx_discovery)
             lifx_discovery.start(listen_ip=ip)
-            tasks.append(self.loop.create_task(manager.lifx_ip()))
+            tasks.append(aio.create_task(manager.lifx_ip()))
 
         (done, pending) = await aio.wait(tasks, timeout=timeout)
 
